//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#pragma pack( push, 8 )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#pragma comment( lib, "PServer" )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define PSERVERLIB extern "C" __declspec( dllimport )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Connection structure.
//
typedef struct _CONNECTION{
    SOCKET Socket;
    DWORD  dwIP;
    PVOID  pvPerConnectionUserData;
}CONNECTION, *PCONNECTION;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// I/O completion structure.
//
typedef struct _IO_DATA{
    DWORD dwBytesTransferred;
    PVOID pvPerIoUserData;
}IO_DATA, *PIO_DATA;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Information about the server status.
//
typedef struct _INFORMATION{
    DWORD dwCpuUsage;
    DWORD dwConnectionsCount;
    DWORD dwOutgoingConnectionsCount;
    DWORD dwWorkerThreadsCount;
    DWORD dwWorkerThreadsBusyCount;
    DWORD dwIoCompletionLen;
    DWORD dwUserThreadsCount;
    DWORD dwUserThreadsBusyCount;
    DWORD dwUserIoCompletionLen;
}INFORMATION, *PINFORMATION;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// Callback functions.
//
typedef BOOL (CALLBACK *ONCONNECTPROC)( PCONNECTION pConnection );
typedef VOID (CALLBACK *ONDISCONNECTPROC)( PCONNECTION pConnection );
typedef VOID (CALLBACK *ONREMOVECONNECTIONPROC)( PCONNECTION pConnection );
typedef BOOL (CALLBACK *ONRECVPROC)( PCONNECTION pConnection, PIO_DATA pIoData );
typedef BOOL (CALLBACK *ONSENDPROC)( PCONNECTION pConnection, PIO_DATA pIoData );
typedef BOOL (CALLBACK *ONOUTGOINGCONNECTPROC)( PCONNECTION pConnection, DWORD dwNtStatus );
typedef VOID (CALLBACK *ONERRORPROC)( PCWSTR pszFunc, DWORD dwError );
typedef BOOL (CALLBACK *WORKERTHREADADDITIONHANDLERPROC)( PINFORMATION pInformation );
typedef BOOL (CALLBACK *WORKERTHREADREMOVALHANDLERPROC)( PINFORMATION pInformation );
typedef BOOL (CALLBACK *USERTHREADADDITIONHANDLERPROC)( PINFORMATION pInformation );
typedef BOOL (CALLBACK *USERTHREADREMOVALHANDLERPROC)( PINFORMATION pInformation );
typedef VOID (CALLBACK *USERPOOLHANDLERPROC)( DWORD dwMessage, PCONNECTION pConnection, PIO_DATA pIoData );
typedef BOOL (CALLBACK *ENUMCONNECTIONSPROC)( PCONNECTION pConnection, PVOID pvParam );
typedef VOID (CALLBACK *SOCKETCREATIONHANDLERPROC)( SOCKET Socket, PVOID pvParam );

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Server initialization structure.
//
typedef struct _PSERVERPARAM{
    DWORD dwWorkerThreadsMin;
    DWORD dwWorkerThreadsMax;
    DWORD dwWorkerThreadsIdleTimeout;
    DWORD dwUserThreadsMin;
    DWORD dwUserThreadsMax;
    DWORD dwUserThreadsIdleTimeout;
    DWORD dwCpuUsageMin;
    DWORD dwCpuUsageMax;
    DWORD dwMultipleIoReqsMax;
    DWORD dwPort;
    DWORD dwInterface;
    DWORD dwSocketRecvBufferSize;
    DWORD dwSocketSendBufferSize;
    DWORD dwKeepAliveTime;
    DWORD dwKeepAliveInterval;
    DWORD dwConnectionsIdleTimeout;
    DWORD dwInitAccepts;

    struct _CALLBACKS{
        ONCONNECTPROC                   OnConnect;
        ONDISCONNECTPROC                OnDisconnect;
        ONREMOVECONNECTIONPROC          OnRemoveConnection;
        ONSENDPROC                      OnSend;
        ONRECVPROC                      OnRecv;
        ONOUTGOINGCONNECTPROC           OnOutgoingConnect;
        ONERRORPROC                     OnError;
        WORKERTHREADADDITIONHANDLERPROC WorkerThreadAdditionHandler;
        WORKERTHREADREMOVALHANDLERPROC  WorkerThreadRemovalHandler;
        USERTHREADADDITIONHANDLERPROC   UserThreadAdditionHandler;
        USERTHREADREMOVALHANDLERPROC    UserThreadRemovalHandler;
        USERPOOLHANDLERPROC             UserPoolHandler;
        }CALLBACKS;
}PSERVERPARAM, *PPSERVERPARAM;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// Server functions.
//
PSERVERLIB DWORD WINAPI PServer_Start( PPSERVERPARAM pParam );
PSERVERLIB VOID  WINAPI PServer_Stop();
PSERVERLIB DWORD WINAPI PServer_PostRecv( PCONNECTION pConnection, PBYTE pbBuffer, DWORD dwSize, PVOID pvPerIoUserData );
PSERVERLIB DWORD WINAPI PServer_PostSend( PCONNECTION pConnection, PBYTE pbBuffer, DWORD dwSize, PVOID pvPerIoUserData, BOOL fCompletionRequired );
PSERVERLIB VOID  WINAPI PServer_EnableConnectionIdleCheck( PCONNECTION pConnection, BOOL fEnable );
PSERVERLIB VOID  WINAPI PServer_CloseConnection( PCONNECTION pConnection );
PSERVERLIB VOID  WINAPI PServer_QueryInformation( PINFORMATION pInformation );
PSERVERLIB VOID  WINAPI PServer_EnumConnections( ENUMCONNECTIONSPROC pEnumConnectionsProc, PVOID pvParam );
PSERVERLIB DWORD WINAPI PServer_PostMessageToUserPool( DWORD dwMessage, PCONNECTION pConnection, PIO_DATA pIoData );
PSERVERLIB DWORD WINAPI PServer_Connect( PSTR pszServer, DWORD dwPort, PVOID pvParam, SOCKETCREATIONHANDLERPROC pSocketCreationHandler );

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#pragma pack( pop )

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////